﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;


namespace FixPoseChannels
{
    enum ParseState
    {
        LookForActor,
        LookForActorName,
        LookForChannels,
        LookForChannelName,
        LookForChannelEnd,
        LookForLink,
        GetFigure,
        GetActor,
        GetChannel
    }

    class FigureParseMachine
    {
        static ParseState _currentState;
        static PoserActor _currentActor;
        static PoserItem _currentChannel;


        public static void Start()
        {
            _currentState = ParseState.LookForActor;
            _currentActor = null;
            _currentChannel = null;
        }


        public static PoserActor ProcessLine(string line)
        {
            switch(_currentState)
            {
                case ParseState.LookForActor:
                    if (line.Contains("actor "))
                    {
                        _currentActor = new PoserActor();
                        _currentActor.InternalName = line.Replace("actor ", "").Trim();
                        int pos = _currentActor.InternalName.IndexOf(':');
                        if (pos > -1) _currentActor.InternalName = _currentActor.InternalName.Remove(pos);
                        _currentState = ParseState.LookForActorName;
                    }
                break;
                // A CR2 gets two entries for each actor.  We want to bail on the first one.
                // We can tell because we won't find the "name" entry.
                case ParseState.LookForActorName:
                    if (line.Contains('}'))
                    {
                        _currentActor = null;
                        _currentState = ParseState.LookForActor;
                    }
                    else if (line.Contains("name "))
                    {
                        _currentActor.DisplayName = line.Replace("name ", "").Trim();
                        _currentState = ParseState.LookForChannels;
                    }
                break;
                case ParseState.LookForChannels:
                    if (line.Contains("targetGeom "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "targetGeom";
                        _currentChannel.InternalName = line.Replace("targetGeom ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("valueParm "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "valueParm";
                        _currentChannel.InternalName = line.Replace("valueParm ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("rotateX "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "rotateX";
                        _currentChannel.InternalName = line.Replace("rotateX ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("rotateY "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "rotateY";
                        _currentChannel.InternalName = line.Replace("rotateY ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("rotateZ "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "rotateZ";
                        _currentChannel.InternalName = line.Replace("rotateZ ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("translateX "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "translateX";
                        _currentChannel.InternalName = line.Replace("translateX ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("translateY "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "translateY";
                        _currentChannel.InternalName = line.Replace("translateY ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("translateZ "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "translateZ";
                        _currentChannel.InternalName = line.Replace("translateZ ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("scale "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "scale";
                        _currentChannel.InternalName = line.Replace("scale ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("scaleX "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "scaleX";
                        _currentChannel.InternalName = line.Replace("scaleX ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("scaleY "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "scaleY";
                        _currentChannel.InternalName = line.Replace("scaleY ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("scaleZ "))
                    {
                        _currentChannel = new PoserItem();
                        _currentChannel.ItemName = "scaleZ";
                        _currentChannel.InternalName = line.Replace("scaleZ ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("actor "))
                    {
                        PoserActor temp = _currentActor;
                        _currentActor = new PoserActor();
                        _currentActor.InternalName = line.Replace("actor ", "").Trim();
                        int pos = _currentActor.InternalName.IndexOf(':');
                        if (pos > -1) _currentActor.InternalName = _currentActor.InternalName.Remove(pos);
                        _currentState = ParseState.LookForActorName;
                        return temp;
                    }
                break;
                case ParseState.LookForChannelName:
                    if (line.Contains("name "))
                    {
                        _currentChannel.DisplayName = line.Replace("name ", "").Trim();
                        _currentActor.Channels.Add(_currentChannel);
                        _currentChannel = null;
                        _currentState = ParseState.LookForChannels;
                    }
                break;
            }

            return null;
        }

        public static PoserActor getLastActor()
        {
            return _currentActor;
        }
    }











    class PoseParseMachine
    {
        static ParseState _currentState;
        static PoseFix _currentFix;


        public static void Start()
        {
            _currentState = ParseState.LookForLink;
            _currentFix = null;
        }


        public static void StartForName()
        {
            _currentState = ParseState.LookForActor;
            _currentFix = null;
        }


        public static PoseFix ProcessLineForName(string line)
        {
            switch (_currentState)
            {
                case ParseState.LookForActor:
                    if (line.Contains("actor "))
                    {
                        _currentFix = new PoseFix();
                        _currentFix.Actor = line.Replace("actor ", "").Trim();
                        int pos = _currentFix.Actor.IndexOf(':');
                        if (pos > -1) _currentFix.Actor = _currentFix.Actor.Remove(pos);
                        _currentState = ParseState.LookForChannels;
                    }
                break;
                case ParseState.LookForChannels:
                    if (line.Contains("targetGeom "))
                    {
                        _currentFix.Channel = line.Replace("targetGeom ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("valueParm "))
                    {
                        _currentFix.Channel = line.Replace("valueParm ", "").Trim();
                        _currentState = ParseState.LookForChannelName;
                    }
                    else if (line.Contains("actor "))
                    {
                        _currentFix.Actor = line.Replace("actor ", "").Trim();
                        int pos = _currentFix.Actor.IndexOf(':');
                        if (pos > -1) _currentFix.Actor = _currentFix.Actor.Remove(pos);
                        _currentState = ParseState.LookForChannels;
                    }
                break;
                case ParseState.LookForChannelName:
                    if (line.Contains("name "))
                    {
                        _currentFix.Name = line.Replace("name ", "").Trim();
                        _currentState = ParseState.LookForChannels;
                        return (_currentFix);
                    }
                break;
            }

            return null;
        }

        
        
        public static PoseFix ProcessLine(string line)
        {
            switch (_currentState)
            {
                case ParseState.LookForLink:
                    if (line.Contains("valueOpPlus") ||
                        line.Contains("valueOpMinus") ||
                        line.Contains("valueOpTimes") ||
                        line.Contains("valueOpDivideBy") ||
                        line.Contains("valueOpDivideInto") ||
                        line.Contains("valueOpDeltaAdd"))
                    {
                        _currentFix = new PoseFix();
                        _currentState = ParseState.GetFigure;
                    }
                break;
                case ParseState.GetFigure:
                    _currentFix.Figure = line.Trim();
                    _currentState = ParseState.GetActor;
                break;
                case ParseState.GetActor:
                    int pos = line.IndexOf(':');
                    if (pos > -1) _currentFix.Actor = line.Remove(pos).Trim();
                    else _currentFix.Actor = line.Trim();
                    _currentState = ParseState.GetChannel;
                break;
                case ParseState.GetChannel:
                    _currentFix.Channel = line.Trim();
                    _currentState = ParseState.LookForLink;
                return (_currentFix);
            }

            return null;
        }
    }




    class ReadScriptResolver
    {
        string _runtimePath;
        string _text;
        StringReader _reader;
        ReadScriptResolver _subResolver;

        public ReadScriptResolver(string RuntimePath, string FileName)
        {
            _runtimePath = RuntimePath;
            FileName = FileName.Remove(0, 1);
            FileName = Path.Combine(RuntimePath, FileName.Replace(':', '\\'));
            _text = File.ReadAllText(FileName);
            _reader = new StringReader(_text);
        }

        public string readLine()
        {
            if(_subResolver != null)
            {
                string subLine = _subResolver.readLine();
                if(subLine != null) return subLine;
                else _subResolver = null;
            }

            string line = _reader.ReadLine();
            while (line != null)
            {
                if (line.Contains("readScript"))
                {
                    _subResolver = new ReadScriptResolver(_runtimePath, line.Replace("readScript ", "").Replace("\"", "").Trim());
                    string subLine = _subResolver.readLine();
                    if (subLine != null) return subLine;
                    else _subResolver = null;
                }

                return line;
            }

            return null;
        }
    }


}
